from fastapi import APIRouter
from notebuild.tool.fastapi import add_api_routes, api_route
from notecoin.strategy.binance import Strategy

path_root = '/home/bingtao/workspace/tmp'


class StrategyServer(APIRouter):
    def __init__(self, prefix='/strategy', *args, **kwargs):
        super(StrategyServer, self).__init__(prefix=prefix, *args, **kwargs)
        self.running = False
        add_api_routes(self)
        self.strategy = Strategy()

    @api_route('/refresh_account', description="get value")
    def refresh_account(self):
        self.strategy.refresh_markets()
        self.strategy.refresh_account()

    @api_route('/buy_market', description="get value")
    def buy_market(self, symbol):
        self.strategy.buy_market(symbol)

    @api_route('/auto_oco', description="get value")
    def auto_oco(self):
        if self.running:
            return {"status": "false", "msg": "is running"}
        self.running = True
        self.strategy.auto_oco()
        self.running = False



from notecoin.base.db import BaseTable
from notedrive.lanzou import LanZouDrive
from sqlalchemy import BIGINT, TIMESTAMP, Column, String, Table, func, select


class LanzouDirectory(BaseTable):
    def __init__(self, table_name="notecoin_lanzou", fid=5679873, *args, **kwargs):
        self.fid = fid
        super(LanzouDirectory, self).__init__(table_name=table_name, *args, **kwargs)
        self.table = Table(self.table_name, self.meta,
                           Column('fid', BIGINT, comment='fid', primary_key=True),
                           Column('gmt_create', TIMESTAMP(True), server_default=func.now()),
                           Column('gmt_modified', TIMESTAMP(True), server_default=func.now()),
                           Column('isfile', BIGINT, comment='是否文件', default='1'),
                           Column('name', String(100), comment='名称', default=''),
                           Column('desc', String(500), comment='描述', default='0'),
                           Column('path', String(500), comment='路径', default=''),
                           Column('downs', BIGINT, comment='下载次数', default='0'),
                           Column('url', String(100), comment='分享链接', default='0'),
                           Column('pwd', String(50), comment='提取码', default='0'),
                           )
        self.create()
        self.drive = LanZouDrive()

        self.drive.login_by_cookie()
        self.drive.ignore_limits()

    def scan_all_file(self, clear=False):
        if clear:
            self.delete_all()
        self._scan_all_file(self.fid, 'notecoin')

    def _scan_all_file(self, fid, path):
        for _dir in self.drive.get_dir_list(fid):
            _path = f'{path}/{_dir.name}'
            if not self.file_exist(_path):
                share = self.drive.get_share_info(fid=_dir.id, is_file=False)
                data = {
                    "fid": _dir.id,
                    "name": _dir.name,
                    "path": _path,
                    "isfile": 0,
                    "desc": share.desc,
                    "url": share.url,
                    "pwd": share.pwd
                }
                self.upsert(value=data)

            if _dir.name.endswith(".tar"):
                continue
            self._scan_all_file(fid=_dir.id, path=_path)

        for _file in self.drive.get_file_list(fid):
            _path = f'{path}/{_file.name}'

            if not self.file_exist(_path):
                share = self.drive.get_share_info(fid=_file.id, is_file=True)
                data = {
                    "fid": _file.id,
                    "name": _file.name,
                    "path": _path,
                    "isfile": 1,
                    "downs": _file.downs,
                    "desc": share.desc,
                    "url": share.url,
                    "pwd": share.pwd
                }
                self.upsert(value=data)

    def file_exist(self, path):
        s = select(self.table.columns).where(self.table.columns.path == path)
        data = [line for line in self.engine.execute(s)]
        return len(data) == 1

    def file_fid(self, path):
        s = select(self.table.columns).where(self.table.columns.path == path)
        data = [line for line in self.engine.execute(s)]

        if len(data) == 1:
            return data[0]['fid']
        else:
            return -1

    def sync(self, path):
        def filter_fun(x): return x.endswith('.csv') or x.startswith("_")

        self.drive.sync_files(path, self.fid, remove_local=True, filter_fun=filter_fun)



import os.path

import oss2
from notefile.compress import tarfile
from notesecret import read_secret

global bucket
bucket = None


def get_bucket():
    global bucket
    if bucket is not None:
        return bucket
    auth = oss2.Auth(
        access_key_id=read_secret("aliyun", "oss", "access_key_id"),
        access_key_secret=read_secret("aliyun", "oss", "access_key_secret"))
    bucket = oss2.Bucket(
        auth,
        endpoint=read_secret("aliyun", "oss", "endpoint"),
        bucket_name=read_secret("aliyun", "oss", "bucket"))


def oss_upload(file1, path='notecoin', tar=False):
    if not tar:
        with open(file1, 'rb') as f:
            bucket.put_object(f"{path}/{os.path.basename(file1)}", f)
        return
    tmp_tar_file = os.path.basename(file1) + ".tar.gz"
    with tarfile.open(tmp_tar_file, "w|xz") as tar:
        tar.add(file1)
    with open(tmp_tar_file, 'rb') as f:
        bucket.put_object(f"{path}/{os.path.basename(tmp_tar_file)}", f)
    os.remove(tmp_tar_file)



import csv
import json
import logging
import time

import ccxt
import pandas as pd
from ccxt.base.exchange import Exchange
from tqdm import tqdm

logger = logging.getLogger()
unix_month = 2678400000
one_hour = 3600 * 1000


class LoadData:
    def __init__(self, exchange: Exchange, csv_path, unix_start, unix_end, *args, **kwargs):
        self.exchange = exchange
        self.csv_path = csv_path
        self.unix_start = unix_start
        self.unix_end = unix_end
        self.cache_data = []

    def load_open(self):
        pass

    def load_all(self, *args, **kwargs):
        self.load_open()
        self.exchange.load_markets()
        pbr = tqdm(self.exchange.symbols)
        for sym in pbr:
            if ':' not in sym:
                pbr.set_description(sym)
                self.load(sym, pbr, *args, **kwargs)
        self.write_data([], False)
        self.load_close()

    def load_close(self):
        pass

    def load(self, symbol, pbr, *args, **kwargs):
        pass

    def write_data(self, data_list, cache=True):
        self.cache_data.extend(data_list)
        if cache and len(self.cache_data) < 10000:
            return
        if len(self.cache_data) == 0:
            return
        df = pd.DataFrame(self.cache_data)
        df = df[(df['timestamp'] >= self.unix_start) & (df['timestamp'] <= self.unix_end)]
        self._write(json.loads(df.to_json(orient='records')))
        self.cache_data.clear()

    def _write(self, data_list):
        pass


class LoadDataKline(LoadData):
    def __init__(self, *args, timeframe='1m', **kwargs):
        super(LoadDataKline, self).__init__(*args, **kwargs)
        self.timeframe = timeframe
        self.csv_file = open(self.csv_path, mode="w")
        self.csv_writer = csv.DictWriter(self.csv_file, delimiter=",",
                                         fieldnames=["symbol", "timestamp", "open", "close", "low", "high", "vol"])
        self.csv_writer.writeheader()

    def _write(self, data_list):
        self.csv_writer.writerows(data_list)
        self.csv_file .flush()

    def load(self, symbol, pbr, *args, **kwargs):
        unix_temp = self.unix_start
        for _ in range(1000):
            if unix_temp >= self.unix_end:
                break
            try:
                result = self.exchange.fetch_ohlcv(symbol, self.timeframe, unix_temp, limit=100)
                result = self.exchange.sort_by(result, 0)
                if len(result) == 0:
                    break
                unix_temp = result[-1][0]
                df = pd.DataFrame(result, columns=['timestamp', 'open', 'close', 'low', 'high', 'vol'])
                df['symbol'] = symbol
                self.write_data(json.loads(df.to_json(orient='records')))
                # time.sleep(int(self.exchange.rateLimit / 1000))
            except Exception as e:
                print(type(e).__name__, str(e))
                self.exchange.sleep(10000)


class LoadTradeKline(LoadData):
    def __init__(self, *args, **kwargs):
        super(LoadTradeKline, self).__init__(*args, **kwargs)
        self.csv_file = open(self.csv_path, mode="w")
        self.csv_writer = csv.DictWriter(self.csv_file, delimiter=",", fieldnames=[
                                         "symbol", "id", "timestamp", "side", "price", "amount"])
        self.csv_writer.writeheader()

    def load_close(self):
        pass

    def _write(self, data_list):
        self.csv_writer.writerows(data_list)
        self.csv_file.flush()

    def load(self, symbol, pbr, *args, **kwargs):
        unix_temp = self.unix_start
        previous_trade_id = None
        for _ in range(10000):
            pbr.set_description(f'{symbol}-{unix_temp}')
            if unix_temp >= self.unix_end:
                break
            try:
                trades = self.exchange.fetch_trades(symbol, unix_temp, limit=1000)
                if len(trades) == 0:
                    unix_temp += one_hour
                    continue
                last_trade = trades[-1]
                if previous_trade_id == last_trade['id']:
                    unix_temp += one_hour
                    continue
                unix_temp = last_trade['timestamp']
                previous_trade_id = last_trade['id']

                result = [{
                    'symbol': trade['symbol'],
                    'id': trade['id'].replace('\n', ''),
                    'timestamp': int(trade['timestamp']),
                    'side': trade['side'][0],
                    'price': trade['price'],
                    'amount': trade['amount'],
                } for trade in trades]

                self.write_data(result)
                #time.sleep(int(self.exchange.rateLimit / 1000))
            except ccxt.NetworkError as e:
                print(type(e).__name__, str(e))
                self.exchange.sleep(1000)



from datetime import datetime, timedelta

from ccxt import binance
from notecoin.coins.base.file import DataFileProperty
from notecoin.task import BaseTask


class LoadTask(BaseTask):
    table_name = 'load'

    def __init__(self, *args, **kwargs):
        self.path_root = '/home/bingtao/workspace/tmp'
        super(LoadTask, self).__init__(*args, **kwargs)

    @staticmethod
    def parse_day(ds):
        ds = ds
        first = datetime.strptime(ds, '%Y-%m-%d') - timedelta(days=1)
        last = first + timedelta(days=1) - timedelta(seconds=1)
        return first, last

    @staticmethod
    def parse_week(ds, weeks=0):
        ds = ds
        first = datetime.strptime(ds, '%Y-%m-%d') + timedelta(weeks=weeks)
        first = first - timedelta(days=first.weekday())
        first = datetime(first.year, first.month, first.day)
        last = first + timedelta(weeks=1) - timedelta(seconds=1)
        return first, last


class LoadKlineDailyTask(LoadTask):
    def refresh(self, ds):
        start, end = self.parse_day(ds)
        file_pro = DataFileProperty(exchange=binance(), path=self.path_root)
        file_pro.file_format = '%Y%m%d'
        file_pro.change_data_type('kline')
        file_pro.change_timeframe('1m')
        file_pro.change_freq('daily')
        file_pro.load_daily(start, end)


class LoadKlineWeeklyTask(LoadTask):
    def refresh(self, ds):
        start, end = self.parse_week(ds, weeks=-1)
        file_pro = DataFileProperty(exchange=binance(), path=self.path_root)
        file_pro.file_format = '%Y%m%d'
        file_pro.change_data_type('kline')
        file_pro.change_timeframe('1m')
        file_pro.change_freq('weekly')
        print((start, end))
        file_pro.load_merge(start, end)


class LoadTradeDailyTask(LoadTask):
    def refresh(self, ds):
        start, end = self.parse_day(ds)
        file_pro = DataFileProperty(exchange=binance(), path=self.path_root)
        file_pro.file_format = '%Y%m%d'
        file_pro.change_data_type('trade')
        file_pro.change_timeframe('detail')
        file_pro.change_freq('daily')
        file_pro.load_daily(start, end)


#LoadKlineWeeklyTask().refresh('2021-9-14')
